import csv
from bitstring import BitArray
from intelhex import IntelHex

stream = BitArray()

with open('input.csv') as csvfile:
    bitread = csv.reader(csvfile, delimiter=',')
    
    for row in bitread:
        s = str(row[2])
        #print s
        stream.append('0b' + s)


error = bool(False)

# look for the preamble defined in the datasheet
preamble_str = '111111110010'
detected_preamble = str(stream[0:12].bin)

if (preamble_str == detected_preamble):
    preamble_valid =  'valid'
else:
    preamble_valid = 'invalid'
    error |= bool(True)
    
print 'Preamble is ' + detected_preamble + ', it is ' + preamble_valid

# read the length field, 24 bit
# this is the total number of clocks to get to the end of the configuration, from the current clock
length_count = stream[12:36].uint
print 'Found total record length ' + str(length_count) + ' bits (' + str(length_count-4) + ' program data bits)'

# define the last bit index we care about
last_index = length_count+4

# read the 4 bit pad between the header and the first section
pad1 = str(stream[36:40].bin)

if (pad1 == '1111'):
    pad1_str = 'valid'
else:
    pad1_str = 'invalid'
    error |= bool(True)

print 'Pad is ' + pad1_str

# attempt to parse the frames for XC3030
# frames are: 1 bit start =0
# 88 data bits
# 3-bit stop bits =1
frame_count = 241
frame_total_size = 92
frame_data_size = frame_total_size - 4
frame_initial_offset = 40

frame_ptr = 0

print 'Reading ' +  str(frame_count) + ' frames from payload section'
last_printed_index = 0
while frame_ptr < frame_count:
    offset = (frame_initial_offset + frame_ptr*frame_total_size)
    frame = stream[offset: offset + frame_total_size]
    if frame[0]:
        print 'Invalid start bit frame ' + str(frame_ptr+1) + ', aborting'
        error |= bool(True)
        break
    if (str(frame[frame_total_size-3: frame_total_size].bin) != '111'):
        print str(frame[frame_total_size-3: frame_total_size])
        print 'Invalid stop bit frame ' + str(frame_ptr+1) + ', aborting'
        error |= bool(True)
        break
    
    #print 'F:' + '%3s' % str(frame_ptr+1) + ' ' + str(frame.bin)
    frame_ptr += 1
    last_printed_index = offset + frame_total_size

postamble = str(stream[last_index - 4: last_index].bin)
if (postamble == '1111'):
    print 'Postamble is valid'
else:
    print 'Postamble is invalid'
    error |= bool(True)

if (error):
    print 'Errors detected, not writing output file'
    exit()
else:
    print 'No errors detected, writing output.bin'
    f = open('output.bin', 'wb')
    #print stream.tobytes()
    f.write(stream.tobytes())
    f.close()
    
    # make a second copy without anything extra at the end
    f = open('output_2.bin', 'wb')
    f.write(stream[:last_index].tobytes())
    #stream.tofile(f)
    f.close()

print 'Generating Intel Hex file'

ih = IntelHex()
ih.loadbin('output.bin')

print 'Loaded'

f = open('output.mcs', 'wb')
ih.tofile(f, format='hex')

print 'Wrote output.mcs'

ih = IntelHex()
ih.loadbin('output_2.bin')

print 'Loaded'

f = open('output_2.mcs', 'wb')
ih.tofile(f, format='hex')

print 'Wrote output_2.mcs'
